## 项目配置
1. `application.properties` 文件中配置
``` java
    # 服务器 HTTP 端口，默认是 8080
    server.port = 9090
    system.name = "springboot演示系统"
```
2. 新建 `SystemCofig.java` 文件，用来映射内容
``` java
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

@Component
@ConfigurationProperties(prefix = "system")
public class SystemConfig {
    private String name;

    public String getName() {
        return name;
    }
    public void setName(String name) {
        this.name = name;
    }
    @Override
    public String toString() {
        return "SystemConfig{" + "name='" + name + '\'' + '}';
    }
}
```
3. IDE 提示 `Spring Boot Configuration Annotation Processor not found in class path`，原因是 `pom.xml` 文件没有添加如下依赖
``` xml
    <dependency>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-configuration-processor</artifactId>
        <optional>true</optional>
    </dependency>
```
4. 新建 `HomeController.java` 文件，用来测试
``` java
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RequestMapping("/home")
@RestController
public class HomeController {
    private final SystemConfig systemConfig;

    @Autowired
    public HomeController(SystemConfig systemConfig) {
        this.systemConfig = systemConfig;
    }

    @GetMapping("/index")
    public String index() {
        String str = this.systemConfig.toString();
        return str;
    }
}
```
5. 启动项目后，在浏览器中访问 `http://localhost:9090/home/index`，返回如下
``` txt
SystemConfig{name='"springbootæ¼ç¤ºç³»ç»"'}
```
6. 第 5 步返回的中文变成乱码了，解决方法：IDE `File | Settings | Editor | File Encodings | Properties Files (*.properties)` ，设置 `Default encoding for properties files | UTF-8`，且勾选 `Transparent native-to-ascii conversion`
7. 重新启动项目，输出如下
``` txt
SystemConfig{name='"springboot演示系统"'}
```

## 自定义文件配置
自定义配置文件的命名不强制 `application` 开头，例如 `system.properties`，须在上面第 2 点使用 `@PropertySource` 注解指定路径，如下示例
``` java
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;
import org.springframework.context.annotation.PropertySource;

@Component
@PropertySource("classpath:system.properties")
@ConfigurationProperties(prefix = "system")
public class SystemConfig { ... }
```

## 多环境化配置 `spring.profile.active`

> 适用于多个环境（如：开发，测试，生产等）时，快速切换配置
1. 主配置文件仍为 `application.properties`
2. 扩展配置文件命名格式为 `application-{profile}.properties`，与主配置文件同目录，其内容如下
    ``` xml
    # {profile} 为配置文件名称
    server.servlet.context-path=/{profile}
    ```
3. `application.properties` 文件配置如下
    ``` xml
    # {profile} 为想要激活的配置文件名称
    spring.profiles.active={profile}
    ```
4. 浏览器访问地址为 `ip:yourPort/{profile}/controller/method`